@@ -18,25 +18,6 @@
 #include "bbox.h"
 #include "wkb.h"
 
-/*      NULL/EMPTY             0 */
-#define SF_Point               1
-#define SF_LineString          2
-#define SF_Polygon             3
-#define SF_MultiPoint          4
-#define SF_MultiLineString     5
-#define SF_MultiPolygon        6
-#define SF_GeometryCollection  7
-#define SF_CircularString      8
-#define SF_CompoundCurve       9
-#define SF_CurvePolygon       10
-#define SF_MultiCurve         11
-#define SF_MultiSurface       12
-#define SF_Curve              13
-#define SF_Surface            14
-#define SF_PolyhedralSurface  15
-#define SF_TIN                16
-#define SF_Triangle           17
-
 #define EWKB_Z_BIT    0x80000000
 #define EWKB_M_BIT    0x40000000
 #define EWKB_SRID_BIT 0x20000000
@@ -417,10 +398,8 @@ unsigned int make_type(const char *cls, const char *dim, bool EWKB = false, int
 		type = SF_TIN;
 	else if (strcmp(cls, "TRIANGLE") == 0)
 		type = SF_Triangle;
-	else {
-		Rcpp::Rcout << cls << " :";
-		throw std::range_error("unknown type!");
-	}
+	else 
+		type = SF_Unknown; // a mix: GEOMETRY
 	if (tp != NULL)
 		*tp = type;
 	if (EWKB) {
@@ -608,11 +587,12 @@ Rcpp::List CPL_write_wkb(Rcpp::List sfc, bool EWKB = false, int endian = 0,
 	// got the following from:
 	// http://stackoverflow.com/questions/24744802/rcpp-how-to-check-if-any-attribute-is-null
 	Rcpp::CharacterVector classes;
-	SEXP sxp = sfc.attr("classes"); // how to check whether an attribute is present w/o using SEXP?
-	if (! Rf_isNull(sxp)) {         // only sfc_GEOMETRY, the mixed bag, sets this
+	bool have_classes = false;
+	if (! Rf_isNull(sfc.attr("classes"))) {         // only sfc_GEOMETRY, the mixed bag, sets this
 		classes = sfc.attr("classes");
 		if (classes.size() != sfc.size())
 			throw std::range_error("attr classes has wrong size: please file an issue");
+		have_classes = true;
 	}
 
 	Rcpp::List crs = sfc.attr("crs"); 
@@ -623,7 +603,7 @@ Rcpp::List CPL_write_wkb(Rcpp::List sfc, bool EWKB = false, int endian = 0,
 	for (int i = 0; i < sfc.size(); i++) {
 		Rcpp::checkUserInterrupt();
 		std::ostringstream os;
-		if (! Rf_isNull(sxp))
+		if (have_classes)
 			cls = classes[i];
 		write_data(os, sfc, i, EWKB, endian, cls, dm, precision, srid);
 		Rcpp::RawVector raw(os.str().size()); // os -> raw:
